#include <lemon/list_graph.h>
#include <lemon/lgf_reader.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <math.h>
#include <vector>

using namespace lemon;
using namespace std;

void readGraph(	string filename, ListGraph& G,
				ListGraph::EdgeMap<int>& label){

	// Le arquivo.
	ifstream file(filename.c_str());
	stringstream input;
	if(file){
		input << file.rdbuf();
		file.close();
	} else {
		cout << "File not found: " << filename << endl;
		exit(1);
	}
	
	// Parser.
	GraphReader<ListGraph>(G, input).edgeMap("label", label).run();
}

void solveEdgeColoring( ListGraph& G,
			 ListGraph::EdgeMap<int>& label, 
			 ListGraph::EdgeMap<int>& coloring){

	map<int, vector<ListGraph::Node> > degreeGroup;
	
	// Agrupa nos por grau e computa grau maximo.
	int max_degree = 0;
	for(ListGraph::NodeIt v(G); v != INVALID; ++v){
		int degree = countOutArcs(G, v);
		if(degree > max_degree){max_degree = degree;}
		(degreeGroup[degree]).push_back(v);
	}

	// Atribui cores comecando do vertice de maior para o de menor grau.
	for(int i=max_degree; i>=0; i--){
		vector<ListGraph::Node> vertices = degreeGroup[i];
		for(int j=0; j < vertices.size(); j++){
			
			ListGraph::Node v = vertices[j];
			// Colore aresta e=(u,v)
			for (ListGraph::IncEdgeIt e(G, v); e != lemon::INVALID; ++e) {
				if(coloring[e] < 1){
				
					// Obtem cores ja utilizadas nas arestas incidentes em u e v.
					set<int> colors;
					for (ListGraph::IncEdgeIt f(G, v); f != lemon::INVALID; ++f) {
						colors.insert(coloring[f]);
					}
					ListGraph::Node u = G.oppositeNode(v, e);
					for (ListGraph::IncEdgeIt f(G, u); f != lemon::INVALID; ++f) {
						colors.insert(coloring[f]);
					}
					
					// Obtem a menor cor disponivel.
					int c = 1;
					while(colors.find(c) != colors.end()){ c++;}
			
					// Atribui cor a aresta.
					coloring[e] = c;
				}
			}
		}
	}
}

int main(int argc, char *argv[]){

	if(argc != 2){
		cout << "uso: " << argv[0] << " <filename>" << endl;
		return 1;
	}

	// Le o grafo.
	string filename = argv[1];	
	
	ListGraph G;
	ListGraph::EdgeMap<int> label(G);	
	
	readGraph(filename, G, label);
	
	// Encontra uma (max_degree+1)-coloracao das arestas.
	ListGraph::EdgeMap<int> coloring(G);
	solveEdgeColoring(G, label, coloring);
	
	// Imprime (max_degree+1)-coloracao.
	for(ListGraph::EdgeIt e(G); e != INVALID; ++e){
		cout << label[e] << "\t" << coloring[e] << "\n";
	}

	return 0;
}
